
From: "Andi Kleen" <ak@suse.de>

Ported from i386/Linus

Fix another TF corner case.  Need to do the special TF handling for all
signals to make debuggers happy

Signed-off-by: Andi Kleen <ak@suse.de>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/arch/x86_64/kernel/signal.c |   12 ++++++++++++
 1 files changed, 12 insertions(+)

diff -puN arch/x86_64/kernel/signal.c~x86_64-fix-interaction-of-single-stepping-with-debuggers arch/x86_64/kernel/signal.c
--- 25/arch/x86_64/kernel/signal.c~x86_64-fix-interaction-of-single-stepping-with-debuggers	Thu Apr  7 15:10:49 2005
+++ 25-akpm/arch/x86_64/kernel/signal.c	Thu Apr  7 15:10:49 2005
@@ -368,6 +368,18 @@ handle_signal(unsigned long sig, siginfo
 		}
 	}
 
+	/*
+	 * If TF is set due to a debugger (PT_DTRACE), clear the TF
+	 * flag so that register information in the sigcontext is
+	 * correct.
+	 */
+	if (unlikely(regs->eflags & TF_MASK)) {
+		if (likely(current->ptrace & PT_DTRACE)) {
+			current->ptrace &= ~PT_DTRACE;
+			regs->eflags &= ~TF_MASK;
+		}
+	}
+
 #ifdef CONFIG_IA32_EMULATION
 	if (test_thread_flag(TIF_IA32)) {
 		if (ka->sa.sa_flags & SA_SIGINFO)
_
